import java.util.Random;
/**
 * Methodenbox für Kapitel 1.5, Aufgabe 4 - Autorennen
 * 
 * @author Franz Jetzinger
 * @version 1.0
 */
public class Methodenbox
{
    private Mittelstreifen mittelstreifen;
    private long startzeit;
    private Auto auto;
    private Blume blume;
    private Random zufall;
    private Regentropfen regentropfen;
    /**
     * Konstruktor für Objekte der Klasse AlgoWrapper
     */
    public Methodenbox()
    {
        zufall = new Random();
        Rechteck fahrbahn = new Rechteck();
        fahrbahn.GrößeSetzen(400,600);
        fahrbahn.FarbeSetzen("grau");
        mittelstreifen = new Mittelstreifen();
        auto = new Auto();
        blume = new Blume();
        setzeBlumePosition(300,50);
        regentropfen = new Regentropfen();
        regentropfen.PositionSetzen(100,20);
    }

    /**
     * Setzt die Position der Blume.
     * @param x neue x-Position
     * @param y neue y-Position
     */
    void setzeBlumePosition(int x, int y)
    {
        blume.PositionSetzen(x,y);
    }
    
    /**
     * Setzt die Sichtbarkeit der Blume
     */
    void setzeBlumeSichtbarkeit(boolean sichtbar)
    {
        blume.SichtbarkeitSetzen(sichtbar);
    }
    
    /**
     * Setzt die Sichtbarkeit des Regentropfens
     */
    void setzeRegentropfenSichtbarkeit(boolean sichtbar)
    {
        regentropfen.SichtbarkeitSetzen(sichtbar);
    }

    /**
     * Setzt die Position des Regentropfens neu.
     * @param x neue x-Position
     * @param y neue y-Position
     */
    void setzeRegentropfenPosition(int x, int y)
    {
        regentropfen.PositionSetzen(x,y);
    }

    /**
     * Gibt eine Zufallszahl im Intervall zwischen von (inkludiert) bis (exkludiert) zurück.
     * @param von Anfang des Intervalls (inklusive)
     * @param bis Ende des Intervalls (exkusive)
     */
    int gibZufallszahl(int von, int bis)
    {
        return zufall.nextInt(bis-von) + von;
    }

    /**
     * Gibt die Y Position der Blume zurück.
     */
    int gibYPositionBlume()
    {
        return blume.YPositionGeben();
    }

    /**
     * Gibt die Y Position des Regentropfens zurück.
     */
    int gibYPositionRegentropfen()
    {
        return regentropfen.YPositionGeben();
    }

    /**
     * Bewegt die Blume um die angegebene anzahl an Schritten in y-Richtung.
     * @param schritte Größe der zurückzulegenden Strecke
     */
    void bewegeBlume(int schritte)
    {
        blume.PositionSetzen(blume.XPositionGeben(),blume.YPositionGeben() + schritte);
    }

    /**
     * Bewegt den Regentropfen um die angegebene anzahl an Schritten in y-Richtung.
     * @param schritte Größe der zurückzulegenden Strecke
     */
    void bewegeRegentropfen(int schritte)
    {
        regentropfen.PositionSetzen(regentropfen.XPositionGeben(),regentropfen.YPositionGeben() + schritte);
    }

    /**
     * Liefert die Zeit seit beginn des Starten der Stoppuhr zurück.
     */
    long gibAktuelleZeitStoppuhr()
    {
        return (System.currentTimeMillis()-startzeit)/1000;
    }

    /**
     * Startet die Stoppuhr.
     */
    void starteStoppuhr()
    {
        startzeit = System.currentTimeMillis();   
    }

    /**
     * Rundet eine Kommazahl auf eine ganze Zahl.
     * @paran zahl Zahl, die gerundet werden soll
     */
    int rundeAufGanzeZahl(long zahl)
    {
        return Math.round(zahl);
    }

    /**
     * Gibt die X Position des Mittelstreifens zurück.
     */
    int gibXPositionMittelstreifen()
    {
        return mittelstreifen.XPositionGeben();
    }

    /**
     * Gibt die Y Position des Mittelstreifens zurück.
     */
    int gibYPositionMittelstreifen()
    {
        return mittelstreifen.YPositionGeben();
    }

    /**
     * Bewegt den Mittelstreifen um die angegebene anzahl an Schritten in y-Richtung.
     * @param schritte Größe der zurückzulegenden Strecke
     */
    void bewegeMittelstreifen(int schritte)
    {
        mittelstreifen.bewege(schritte);
        try {
            Thread.sleep(50);
        } catch (InterruptedException e) {
            System.out.println("Thread is interrupted");
        }
    }

    /**
     * Setzt die Position des Mittelstreifens neu.
     * @param x neue x-Position
     * @param y neue y-Position
     */
    void setzePositionMittelstreifen(int x, int y)
    {
        mittelstreifen.PositionSetzen(x,y);
    }

    /**
     * Prüft, ob das Auto die Blume berührt.
     * @return true, wenn eine Berührung vorliegt, false sonst
     */
    boolean berührtBlume()
    {
        return auto.Berührt(blume);
    }

    /**
     * Prüft, ob das Auto einen Regentropfen berührt.
     * @return true, wenn eine Berührung vorliegt, false sonst
     */
    boolean berührtRegentropfen()
    {
        return auto.Berührt(regentropfen);
    }
    /**
     * Klasse für einen Mittelstreifen
     */
    private class Mittelstreifen extends Figur
    {
        int geschwindigkeit;

        Mittelstreifen()
        {
            super.FigurteilFestlegenRechteck(-50,-10, 20, 100, "weiss");
            super.PositionSetzen(200, 10);
        }

        void setzeGeschwindigkeit(int neueGeschwindigkeit)
        {
            geschwindigkeit = neueGeschwindigkeit;
        }

        void bewege(int schritte)
        {
            PositionSetzen(XPositionGeben(), YPositionGeben() + schritte);
        }

    }
    /**
     * Klasse für ein Auto
     */
    private class Auto extends Figur
    {
        Auto()
        {
            EigeneFigurLöschen();
            this.FigurteilFestlegenRechteck(-50, -40, 20, 10, "schwarz");
            this.FigurteilFestlegenRechteck(-50, 30, 20, 10, "schwarz");
            this.FigurteilFestlegenRechteck(30, -40, 20, 10, "schwarz");
            this.FigurteilFestlegenRechteck(30, 30, 20, 10, "schwarz");
            this.FigurteilFestlegenEllipse(-50, -40, 100, 80, "rot");
            PositionSetzen(200,450);
            this.Drehen(90);
        
        }

        void SonderTasteGedrückt(int taste)
        {
            switch(taste)
            {
                case 37:
                    PositionSetzen(XPositionGeben() - 10, YPositionGeben());
                    break;
                case 39:
                    PositionSetzen(XPositionGeben() + 10,YPositionGeben());
            }
        }

    }
    /**
     * KLasse für einen Regentropfen
     */
    private class Regentropfen extends Figur
    {
        Regentropfen()
        {
            EigeneFigurLöschen();
            FigurteilFestlegenDreieck(-50, 0, 30, -30, 30, 30, "blau");
            FigurteilFestlegenEllipse(10, -30, 40, 60, "blau");
            WinkelSetzen(270);
        }

    }
    /**
     * Klasse für eine Blume
     */
    private class Blume extends Figur
    {
        Blume ()
        {

            EigeneFigurLöschen();
            FigurteilFestlegenEllipse(0, 0, 25, 25, "gelb");    
            FigurteilFestlegenEllipse(0, -20, 20, 20, "blau");
            FigurteilFestlegenEllipse(20, -10, 20, 20, "blau");
            FigurteilFestlegenEllipse(20, 10, 20, 20, "blau");
            FigurteilFestlegenEllipse(0, 20, 20, 20, "blau");
            FigurteilFestlegenEllipse(-20, -10, 20, 20, "blau");
            FigurteilFestlegenEllipse(-20, 10, 20, 20, "blau");
            PositionSetzen(225, 125);
            GanzNachVornBringen();
        }

    }
}

